@@ -24,17 +24,12 @@
 #include "tool_setup.h"
 #include "tool_operate.h"
 
-#include "strcase.h"
-
-#define ENABLE_CURLX_PRINTF
-/* use our own printf() functions */
-#include "curlx.h"
-
+#include <curlx.h>
 #include "tool_cfgable.h"
 #include "tool_doswin.h"
 #include "tool_operhlp.h"
-
-#include "memdebug.h" /* keep this as LAST include */
+#include "tool_msgs.h"
+#include <memdebug.h> /* keep this as LAST include */
 
 void clean_getout(struct OperationConfig *config)
 {
@@ -44,10 +39,10 @@ void clean_getout(struct OperationConfig *config)
 
     while(node) {
       next = node->next;
-      Curl_safefree(node->url);
-      Curl_safefree(node->outfile);
-      Curl_safefree(node->infile);
-      Curl_safefree(node);
+      tool_safefree(node->url);
+      tool_safefree(node->outfile);
+      tool_safefree(node->infile);
+      tool_safefree(node);
       node = next;
     }
     config->url_list = NULL;
@@ -67,8 +62,7 @@ bool output_expected(const char *url, const char *uploadfile)
 
 bool stdin_upload(const char *uploadfile)
 {
-  return (!strcmp(uploadfile, "-") ||
-          !strcmp(uploadfile, ".")) ? TRUE : FALSE;
+  return !strcmp(uploadfile, "-") || !strcmp(uploadfile, ".");
 }
 
 /* Convert a CURLUcode into a CURLcode */
@@ -86,7 +80,7 @@ CURLcode urlerr_cvt(CURLUcode ucode)
 }
 
 /*
- * Adds the file name to the URL if it doesn't already have one.
+ * Adds the filename to the URL if it does not already have one.
  * url will be freed before return if the returned pointer is different
  */
 CURLcode add_file_name_to_url(CURL *curl, char **inurlp, const char *filename)
@@ -95,6 +89,7 @@ CURLcode add_file_name_to_url(CURL *curl, char **inurlp, const char *filename)
   CURLUcode uerr;
   CURLU *uh = curl_url();
   char *path = NULL;
+  char *query = NULL;
   if(uh) {
     char *ptr;
     uerr = curl_url_set(uh, CURLUPART_URL, *inurlp,
@@ -108,16 +103,22 @@ CURLcode add_file_name_to_url(CURL *curl, char **inurlp, const char *filename)
       result = urlerr_cvt(uerr);
       goto fail;
     }
-
+    uerr = curl_url_get(uh, CURLUPART_QUERY, &query, 0);
+    if(!uerr && query) {
+      curl_free(query);
+      curl_free(path);
+      curl_url_cleanup(uh);
+      return CURLE_OK;
+    }
     ptr = strrchr(path, '/');
     if(!ptr || !*++ptr) {
-      /* The URL path has no file name part, add the local file name. In order
+      /* The URL path has no filename part, add the local filename. In order
          to be able to do so, we have to create a new URL in another buffer.*/
 
       /* We only want the part of the local path that is on the right
          side of the rightmost slash and backslash. */
       const char *filep = strrchr(filename, '/');
-      char *file2 = strrchr(filep?filep:filename, '\\');
+      char *file2 = strrchr(filep ? filep : filename, '\\');
       char *encfile;
 
       if(file2)
@@ -127,7 +128,7 @@ CURLcode add_file_name_to_url(CURL *curl, char **inurlp, const char *filename)
       else
         filep = filename;
 
-      /* URL encode the file name */
+      /* URL encode the filename */
       encfile = curl_easy_escape(curl, filep, 0 /* use strlen */);
       if(encfile) {
         char *newpath;
@@ -163,7 +164,7 @@ CURLcode add_file_name_to_url(CURL *curl, char **inurlp, const char *filename)
       /* nothing to do */
       result = CURLE_OK;
   }
-  fail:
+fail:
   curl_url_cleanup(uh);
   curl_free(path);
   return result;
@@ -173,9 +174,9 @@ CURLcode add_file_name_to_url(CURL *curl, char **inurlp, const char *filename)
  * Returns a pointer to a heap-allocated string or NULL if
  * no name part, at location indicated by first argument.
  */
-CURLcode get_url_file_name(char **filename, const char *url)
+CURLcode get_url_file_name(struct GlobalConfig *global,
+                           char **filename, const char *url)
 {
-  const char *pc, *pc2;
   CURLU *uh = curl_url();
   char *path = NULL;
   CURLUcode uerr;
@@ -188,31 +189,42 @@ CURLcode get_url_file_name(char **filename, const char *url)
   uerr = curl_url_set(uh, CURLUPART_URL, url, CURLU_GUESS_SCHEME);
   if(!uerr) {
     uerr = curl_url_get(uh, CURLUPART_PATH, &path, 0);
+    curl_url_cleanup(uh);
+    uh = NULL;
     if(!uerr) {
-      curl_url_cleanup(uh);
+      int i;
+      char *pc = NULL, *pc2 = NULL;
+      for(i = 0; i < 2; i++) {
+        pc = strrchr(path, '/');
+        pc2 = strrchr(pc ? pc + 1 : path, '\\');
+        if(pc2)
+          pc = pc2;
+        if(pc && !pc[1] && !i) {
+          /* if the path ends with slash, try removing the trailing one
+             and get the last directory part */
+          *pc = 0;
+        }
+      }
 
-      pc = strrchr(path, '/');
-      pc2 = strrchr(pc ? pc + 1 : path, '\\');
-      if(pc2)
-        pc = pc2;
-
-      if(pc)
+      if(pc) {
         /* duplicate the string beyond the slash */
-        pc++;
-      else
-        /* no slash => empty string */
-        pc = "";
+        *filename = strdup(pc + 1);
+      }
+      else {
+        /* no slash => empty string, use default */
+        *filename = strdup("curl_response");
+        warnf(global, "No remote file name, uses \"%s\"", *filename);
+      }
 
-      *filename = strdup(pc);
       curl_free(path);
       if(!*filename)
         return CURLE_OUT_OF_MEMORY;
 
-#if defined(MSDOS) || defined(WIN32)
+#if defined(_WIN32) || defined(MSDOS)
       {
         char *sanitized;
         SANITIZEcode sc = sanitize_file_name(&sanitized, *filename, 0);
-        Curl_safefree(*filename);
+        tool_safefree(*filename);
         if(sc) {
           if(sc == SANITIZE_ERR_OUT_OF_MEMORY)
             return CURLE_OUT_OF_MEMORY;
@@ -220,25 +232,8 @@ CURLcode get_url_file_name(char **filename, const char *url)
         }
         *filename = sanitized;
       }
-#endif /* MSDOS || WIN32 */
+#endif /* _WIN32 || MSDOS */
 
-      /* in case we built debug enabled, we allow an environment variable
-       * named CURL_TESTDIR to prefix the given file name to put it into a
-       * specific directory
-       */
-#ifdef DEBUGBUILD
-      {
-        char *tdir = curlx_getenv("CURL_TESTDIR");
-        if(tdir) {
-          char *alt = aprintf("%s/%s", tdir, *filename);
-          Curl_safefree(*filename);
-          *filename = alt;
-          curl_free(tdir);
-          if(!*filename)
-            return CURLE_OUT_OF_MEMORY;
-        }
-      }
-#endif
       return CURLE_OK;
     }
   }
