@@ -25,7 +25,7 @@
 
 #include "memdebug.h"
 
-int test(char *URL)
+CURLcode test(char *URL)
 {
   CURLcode ret = CURLE_OK;
   CURL *curl = NULL;
@@ -39,24 +39,26 @@ int test(char *URL)
     curl_easy_setopt(curl, CURLOPT_URL, URL);
     ret = curl_easy_perform(curl);
     if(ret) {
-      fprintf(stderr, "%s:%d curl_easy_perform() failed with code %d (%s)\n",
-          __FILE__, __LINE__, ret, curl_easy_strerror(ret));
+      curl_mfprintf(stderr,
+                    "%s:%d curl_easy_perform() failed with code %d (%s)\n",
+                    __FILE__, __LINE__, ret, curl_easy_strerror(ret));
       goto test_cleanup;
     }
     curl_easy_getinfo(curl, CURLINFO_REDIRECT_URL, &follow_url);
     curl_easy_getinfo(curl, CURLINFO_RETRY_AFTER, &retry_after);
-    printf("Retry-After %" CURL_FORMAT_CURL_OFF_T "\n", retry_after);
+    curl_mprintf("Retry-After %" CURL_FORMAT_CURL_OFF_T "\n", retry_after);
     curl_easy_setopt(curl, CURLOPT_URL, follow_url);
     ret = curl_easy_perform(curl);
     if(ret) {
-      fprintf(stderr, "%s:%d curl_easy_perform() failed with code %d (%s)\n",
-          __FILE__, __LINE__, ret, curl_easy_strerror(ret));
+      curl_mfprintf(stderr,
+                    "%s:%d curl_easy_perform() failed with code %d (%s)\n",
+                    __FILE__, __LINE__, ret, curl_easy_strerror(ret));
       goto test_cleanup;
     }
 
     curl_easy_reset(curl);
     curl_easy_getinfo(curl, CURLINFO_RETRY_AFTER, &retry_after);
-    printf("Retry-After %" CURL_FORMAT_CURL_OFF_T "\n", retry_after);
+    curl_mprintf("Retry-After %" CURL_FORMAT_CURL_OFF_T "\n", retry_after);
   }
 
 test_cleanup:
@@ -65,4 +67,3 @@ test_cleanup:
 
   return ret;
 }
-
