@@ -30,6 +30,7 @@
    knowledge about the system we're building this on */
 
 #define CURL_NO_OLDIES
+#define CURL_DISABLE_DEPRECATION 1
 
 #include "curl_setup.h"
 
@@ -142,7 +143,7 @@ extern int unitfail;
 
 #define exe_easy_init(A,Y,Z) do {                                 \
   if(((A) = curl_easy_init()) == NULL) {                          \
-    fprintf(stderr, "%s:%d curl_easy_init() failed\n", (Y), (Z)); \
+    curl_mfprintf(stderr, "%s:%d curl_easy_init() failed\n", (Y), (Z)); \
     res = TEST_ERR_EASY_INIT;                                     \
   }                                                               \
 } while(0)
@@ -163,7 +164,7 @@ extern int unitfail;
 
 #define exe_multi_init(A,Y,Z) do {                                 \
   if(((A) = curl_multi_init()) == NULL) {                          \
-    fprintf(stderr, "%s:%d curl_multi_init() failed\n", (Y), (Z)); \
+    curl_mfprintf(stderr, "%s:%d curl_multi_init() failed\n", (Y), (Z)); \
     res = TEST_ERR_MULTI;                                          \
   }                                                                \
 } while(0)
@@ -185,7 +186,7 @@ extern int unitfail;
 #define exe_easy_setopt(A,B,C,Y,Z) do {                    \
   CURLcode ec;                                             \
   if((ec = curl_easy_setopt((A), (B), (C))) != CURLE_OK) { \
-    fprintf(stderr, "%s:%d curl_easy_setopt() failed, "    \
+    curl_mfprintf(stderr, "%s:%d curl_easy_setopt() failed, "    \
             "with code %d (%s)\n",                         \
             (Y), (Z), (int)ec, curl_easy_strerror(ec));    \
     res = ec;                                              \
@@ -209,7 +210,7 @@ extern int unitfail;
 #define exe_multi_setopt(A, B, C, Y, Z) do {                \
   CURLMcode ec;                                             \
   if((ec = curl_multi_setopt((A), (B), (C))) != CURLM_OK) { \
-    fprintf(stderr, "%s:%d curl_multi_setopt() failed, "    \
+    curl_mfprintf(stderr, "%s:%d curl_multi_setopt() failed, "    \
             "with code %d (%s)\n",                          \
             (Y), (Z), (int)ec, curl_multi_strerror(ec));    \
     res = TEST_ERR_MULTI;                                   \
@@ -233,7 +234,7 @@ extern int unitfail;
 #define exe_multi_add_handle(A,B,Y,Z) do {                   \
   CURLMcode ec;                                              \
   if((ec = curl_multi_add_handle((A), (B))) != CURLM_OK) {   \
-    fprintf(stderr, "%s:%d curl_multi_add_handle() failed, " \
+    curl_mfprintf(stderr, "%s:%d curl_multi_add_handle() failed, " \
             "with code %d (%s)\n",                           \
             (Y), (Z), (int)ec, curl_multi_strerror(ec));     \
     res = TEST_ERR_MULTI;                                    \
@@ -257,7 +258,7 @@ extern int unitfail;
 #define exe_multi_remove_handle(A,B,Y,Z) do {                   \
   CURLMcode ec;                                                 \
   if((ec = curl_multi_remove_handle((A), (B))) != CURLM_OK) {   \
-    fprintf(stderr, "%s:%d curl_multi_remove_handle() failed, " \
+    curl_mfprintf(stderr, "%s:%d curl_multi_remove_handle() failed, " \
             "with code %d (%s)\n",                              \
             (Y), (Z), (int)ec, curl_multi_strerror(ec));        \
     res = TEST_ERR_MULTI;                                       \
@@ -282,13 +283,13 @@ extern int unitfail;
 #define exe_multi_perform(A,B,Y,Z) do {                          \
   CURLMcode ec;                                                  \
   if((ec = curl_multi_perform((A), (B))) != CURLM_OK) {          \
-    fprintf(stderr, "%s:%d curl_multi_perform() failed, "        \
+    curl_mfprintf(stderr, "%s:%d curl_multi_perform() failed, "        \
             "with code %d (%s)\n",                               \
             (Y), (Z), (int)ec, curl_multi_strerror(ec));         \
     res = TEST_ERR_MULTI;                                        \
   }                                                              \
   else if(*((B)) < 0) {                                          \
-    fprintf(stderr, "%s:%d curl_multi_perform() succeeded, "     \
+    curl_mfprintf(stderr, "%s:%d curl_multi_perform() succeeded, "     \
             "but returned invalid running_handles value (%d)\n", \
             (Y), (Z), (int)*((B)));                              \
     res = TEST_ERR_NUM_HANDLES;                                  \
@@ -312,13 +313,13 @@ extern int unitfail;
 #define exe_multi_fdset(A, B, C, D, E, Y, Z) do {                    \
   CURLMcode ec;                                                      \
   if((ec = curl_multi_fdset((A), (B), (C), (D), (E))) != CURLM_OK) { \
-    fprintf(stderr, "%s:%d curl_multi_fdset() failed, "              \
+    curl_mfprintf(stderr, "%s:%d curl_multi_fdset() failed, "              \
             "with code %d (%s)\n",                                   \
             (Y), (Z), (int)ec, curl_multi_strerror(ec));             \
     res = TEST_ERR_MULTI;                                            \
   }                                                                  \
   else if(*((E)) < -1) {                                             \
-    fprintf(stderr, "%s:%d curl_multi_fdset() succeeded, "           \
+    curl_mfprintf(stderr, "%s:%d curl_multi_fdset() succeeded, "           \
             "but returned invalid max_fd value (%d)\n",              \
             (Y), (Z), (int)*((E)));                                  \
     res = TEST_ERR_NUM_HANDLES;                                      \
@@ -342,13 +343,13 @@ extern int unitfail;
 #define exe_multi_timeout(A,B,Y,Z) do {                      \
   CURLMcode ec;                                              \
   if((ec = curl_multi_timeout((A), (B))) != CURLM_OK) {      \
-    fprintf(stderr, "%s:%d curl_multi_timeout() failed, "    \
+    curl_mfprintf(stderr, "%s:%d curl_multi_timeout() failed, "    \
             "with code %d (%s)\n",                           \
             (Y), (Z), (int)ec, curl_multi_strerror(ec));     \
     res = TEST_ERR_BAD_TIMEOUT;                              \
   }                                                          \
   else if(*((B)) < -1L) {                                    \
-    fprintf(stderr, "%s:%d curl_multi_timeout() succeeded, " \
+    curl_mfprintf(stderr, "%s:%d curl_multi_timeout() succeeded, " \
             "but returned invalid timeout value (%ld)\n",    \
             (Y), (Z), (long)*((B)));                         \
     res = TEST_ERR_BAD_TIMEOUT;                              \
@@ -372,13 +373,13 @@ extern int unitfail;
 #define exe_multi_poll(A,B,C,D,E,Y,Z) do {                          \
   CURLMcode ec;                                                     \
   if((ec = curl_multi_poll((A), (B), (C), (D), (E))) != CURLM_OK) { \
-    fprintf(stderr, "%s:%d curl_multi_poll() failed, "              \
+    curl_mfprintf(stderr, "%s:%d curl_multi_poll() failed, "              \
             "with code %d (%s)\n",                                  \
             (Y), (Z), (int)ec, curl_multi_strerror(ec));            \
     res = TEST_ERR_MULTI;                                           \
   }                                                                 \
   else if(*((E)) < 0) {                                             \
-    fprintf(stderr, "%s:%d curl_multi_poll() succeeded, "           \
+    curl_mfprintf(stderr, "%s:%d curl_multi_poll() succeeded, "           \
             "but returned invalid numfds value (%d)\n",             \
             (Y), (Z), (int)*((E)));                                 \
     res = TEST_ERR_NUM_HANDLES;                                     \
@@ -402,7 +403,7 @@ extern int unitfail;
 #define exe_multi_wakeup(A,Y,Z) do {                     \
   CURLMcode ec;                                          \
   if((ec = curl_multi_wakeup((A))) != CURLM_OK) {        \
-    fprintf(stderr, "%s:%d curl_multi_wakeup() failed, " \
+    curl_mfprintf(stderr, "%s:%d curl_multi_wakeup() failed, " \
             "with code %d (%s)\n",                       \
             (Y), (Z), (int)ec, curl_multi_strerror(ec)); \
     res = TEST_ERR_MULTI;                                \
@@ -427,7 +428,7 @@ extern int unitfail;
     int ec;                                                     \
     if(select_wrapper((A), (B), (C), (D), (E)) == -1) {         \
       ec = SOCKERRNO;                                           \
-      fprintf(stderr, "%s:%d select() failed, with "            \
+      curl_mfprintf(stderr, "%s:%d select() failed, with "            \
               "errno %d (%s)\n",                                \
               (Y), (Z), ec, strerror(ec));                      \
       res = TEST_ERR_SELECT;                                    \
@@ -455,7 +456,7 @@ extern int unitfail;
 #define exe_test_timedout(Y,Z) do {                                       \
   long timediff = tutil_tvdiff(tutil_tvnow(), tv_test_start);             \
   if(timediff > (TEST_HANG_TIMEOUT)) {                                    \
-    fprintf(stderr, "%s:%d ABORTING TEST, since it seems "                \
+    curl_mfprintf(stderr, "%s:%d ABORTING TEST, since it seems "          \
             "that it would have run forever (%ld ms > %ld ms)\n",         \
             (Y), (Z), timediff, (long) (TEST_HANG_TIMEOUT));              \
     res = TEST_ERR_RUNS_FOREVER;                                          \
@@ -479,7 +480,7 @@ extern int unitfail;
 #define exe_global_init(A,Y,Z) do {                     \
   CURLcode ec;                                          \
   if((ec = curl_global_init((A))) != CURLE_OK) {        \
-    fprintf(stderr, "%s:%d curl_global_init() failed, " \
+    curl_mfprintf(stderr, "%s:%d curl_global_init() failed, " \
             "with code %d (%s)\n",                      \
             (Y), (Z), (int)ec, curl_easy_strerror(ec)); \
     res = ec;                                           \
@@ -506,7 +507,7 @@ extern int unitfail;
   CURLcode test(char *URL)                      \
   {                                             \
     (void)URL;                                  \
-    fprintf(stderr, "Missing support\n");       \
+    curl_mfprintf(stderr, "Missing support\n");       \
     return CURLE_UNSUPPORTED_PROTOCOL;          \
   }
 #endif
@@ -524,7 +525,7 @@ extern CURLcode test(char *URL); /* the actual test function provided by each
   CURLcode test(char *URL)                      \
   {                                             \
     (void)URL;                                  \
-    fprintf(stderr, "Missing support\n");       \
+    curl_mfprintf(stderr, "Missing support\n");       \
     return CURLE_UNSUPPORTED_PROTOCOL;          \
   }
 #endif
